@@ -12,7 +12,7 @@ use Perinci::Sub::Util qw(gen_modified_sub);
 our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
 our $DATE = '2022-08-20'; # DATE
 our $DIST = 'App-wordlist'; # DIST
-our $VERSION = '0.289'; # VERSION
+our $VERSION = '0.290'; # VERSION
 
 our %SPEC;
 
@@ -915,27 +915,32 @@ gen_modified_sub(
 
 This is a wrapper to <prog:wordlist> designed to be a convenient helper to solve
 Wordle puzzle. By default it greps from the `EN::Wordle` wordlist. It accepts
-5-character pattern like this:
+a series of guesses in a format like the following:
 
-    wt_S_
+    A^R^isE^
+    Pound
+    might
+    blA^ck
+    PR^ivY^
 
-where lowercase means letter in the incorrect position, uppercase means letter
-in the correct position, and underscore means unguessed. It will convert this
-pattern to regex and the `--chars-unordered` option and pass it to `wordlist`.
+where lowercase means wrong guess, uppercase means correct letter and position,
+while (uppercase) letter followed by a caret (`^`) means the letter exists in
+another position. It will convert these guesses to regex patterns and the
+`--chars-unordered` option and pass it to `wordlist`.
 
 _
         $_[0]{examples} = [
             {
-                argv => ['wt_S_'],
-                summary => 'One guess (lowercase means incorrect position)',
+                argv => ['cR^eEk'],
+                summary => 'One guess',
                 test => 0,
-                'x.doc.show_result' => 1,
+                'x.doc.show_result' => 0,
             },
             {
-                argv => ['___s_', '_o___', '___ht', '_L___'],
-                summary => 'Four guesses (lowercase means incorrect position)',
+                argv => ['A^R^isE^', 'Pound', 'might', 'blA^ck', 'PR^ivY^'],
+                summary => 'Five guesses',
                 test => 0,
-                'x.doc.show_result' => 1,
+                'x.doc.show_result' => 0,
             },
         ];
     },
@@ -945,17 +950,35 @@ _
         $args{arg} //= [];
 
         my $chars_unordered = '';
+        my $possible_letters = join '', "a".."z";
+        my $nonpresent_letters = '';
         my @new_arg;
         for my $arg (@{ $args{arg} }) {
+            my @chars = split //, $arg;
             my $re = '';
-            for my $char (split //, $arg) {
-                if ($char eq '_') {
-                    $re .= '.';
-                } elsif ($char eq uc($char)) {
-                    $re .= quotemeta(lc $char);
-                } else {
-                    $re .= "[^".quotemeta($char)."]";
+            my %letter_exists;
+            while (@chars) {
+                my $char = shift @chars;
+                return [400, "Invalid letter '$char' in guess '$arg'"] unless $char =~ /[A-Za-z]/;
+                my $caret = @chars && $chars[0] eq '^' ? shift(@chars) : '';
+                my $uc = $char eq uc $char;
+                $char = lc $char;
+
+                if ($caret) { # letter is in another position
+                    my $letters = $possible_letters;
+                    $letters =~ s/$char//;
+                    $re .= "[$letters]";
+                    $letter_exists{$char}++;
                     $chars_unordered .= $char unless index($chars_unordered, $char) >= 0;
+                } elsif ($uc) { # correct guess
+                    $re .= $char;
+                    $letter_exists{$char}++;
+                    $chars_unordered .= $char unless index($chars_unordered, $char) >= 0;
+                } else { # wrong guess
+                    my $letters = $possible_letters;
+                    $letters =~ s/$char//;
+                    $possible_letters =~ s/$char// unless $letter_exists{$char};
+                    $re .= "[$letters]";
                 }
             }
             $re = "/\\A$re\\z/";
@@ -985,7 +1008,7 @@ App::wordlist - Grep words from (or test them against) WordList::*
 
 =head1 VERSION
 
-This document describes version 0.289 of App::wordlist (from Perl distribution App-wordlist), released on 2022-08-20.
+This document describes version 0.290 of App::wordlist (from Perl distribution App-wordlist), released on 2022-08-20.
 
 =head1 SYNOPSIS
 
@@ -1246,143 +1269,30 @@ Examples:
 
 =over
 
-=item * One guess (lowercase means incorrect position):
+=item * One guess:
 
- wordlist_wordle(arg => ["wt_S_"]);
+ wordlist_wordle(arg => ["cR^eEk"]);
 
-Result:
+=item * Five guesses:
 
- [
-   200,
-   "OK",
-   sub {
-     REDO: return if $i_wordlist >= @$wordlists;
-     my $wl = $$wordlists[$i_wordlist];
-     unless ($wl_obj) {
-     log_trace("Instantiating wordlist $wl ...");
-     eval {
-     do {
-     $wl_obj = Module::Load::Util::instantiate_class_with_optional_args({'ns_prefix', 'WordList'}, $wl)
-     }
-     };
-     warn if $@;
-     unless ($wl_obj) {
-     ++$i_wordlist;
-     goto REDO;
-     }
-     $nth_word = 0;
-     }
-     my $word = $nth_word++ ? $wl_obj->next_word : $wl_obj->first_word;
-     unless (defined $word) {
-     undef $wl_obj;
-     ++$i_wordlist;
-     goto REDO;
-     }
-     goto REDO if not $random and $num > 0 and $n >= $num;
-     goto REDO if defined $args{'len'} and _length_in_graphemes($word) != $args{'len'};
-     goto REDO if defined $args{'min_len'} and _length_in_graphemes($word) < $args{'min_len'};
-     goto REDO if defined $args{'max_len'} and _length_in_graphemes($word) > $args{'max_len'};
-     goto REDO if defined $args{'chars_unordered'} and not _word_has_chars_unordered($word, $args{'chars_unordered'}, $ci);
-     goto REDO if defined $args{'chars_ordered'} and not _word_has_chars_ordered($word, $args{'chars_ordered'}, $ci);
-     my $cmpword = $ci ? lc $word : $word;
-     my $match_arg;
-     foreach $_ (@$arg) {
-     my $match = ref $_ eq 'Regexp' ? $cmpword =~ /$_/ : index($cmpword, $_) != -1;
-     if ($or) {
-     if ($match) {
-     ++$n;
-     return &$code_format_word($wl, $word, $use_color ? $_ : undef, $ci);
-     }
-     }
-     else {
-     unless ($match) {
-     goto REDO;
-     }
-     }
-     $match_arg = $_;
-     }
-     if (not $or && @$arg) {
-     ++$n;
-     return &$code_format_word($wl, $word, $use_color ? $match_arg : undef, $ci);
-     }
-   },
-   { stream => 1 },
- ]
-
-=item * Four guesses (lowercase means incorrect position):
-
- wordlist_wordle(arg => ["___s_", "_o___", "___ht", "_L___"]);
-
-Result:
-
- [
-   200,
-   "OK",
-   sub {
-     REDO: return if $i_wordlist >= @$wordlists;
-     my $wl = $$wordlists[$i_wordlist];
-     unless ($wl_obj) {
-     log_trace("Instantiating wordlist $wl ...");
-     eval {
-     do {
-     $wl_obj = Module::Load::Util::instantiate_class_with_optional_args({'ns_prefix', 'WordList'}, $wl)
-     }
-     };
-     warn if $@;
-     unless ($wl_obj) {
-     ++$i_wordlist;
-     goto REDO;
-     }
-     $nth_word = 0;
-     }
-     my $word = $nth_word++ ? $wl_obj->next_word : $wl_obj->first_word;
-     unless (defined $word) {
-     undef $wl_obj;
-     ++$i_wordlist;
-     goto REDO;
-     }
-     goto REDO if not $random and $num > 0 and $n >= $num;
-     goto REDO if defined $args{'len'} and _length_in_graphemes($word) != $args{'len'};
-     goto REDO if defined $args{'min_len'} and _length_in_graphemes($word) < $args{'min_len'};
-     goto REDO if defined $args{'max_len'} and _length_in_graphemes($word) > $args{'max_len'};
-     goto REDO if defined $args{'chars_unordered'} and not _word_has_chars_unordered($word, $args{'chars_unordered'}, $ci);
-     goto REDO if defined $args{'chars_ordered'} and not _word_has_chars_ordered($word, $args{'chars_ordered'}, $ci);
-     my $cmpword = $ci ? lc $word : $word;
-     my $match_arg;
-     foreach $_ (@$arg) {
-     my $match = ref $_ eq 'Regexp' ? $cmpword =~ /$_/ : index($cmpword, $_) != -1;
-     if ($or) {
-     if ($match) {
-     ++$n;
-     return &$code_format_word($wl, $word, $use_color ? $_ : undef, $ci);
-     }
-     }
-     else {
-     unless ($match) {
-     goto REDO;
-     }
-     }
-     $match_arg = $_;
-     }
-     if (not $or && @$arg) {
-     ++$n;
-     return &$code_format_word($wl, $word, $use_color ? $match_arg : undef, $ci);
-     }
-   },
-   { stream => 1 },
- ]
+ wordlist_wordle(arg => ["A^R^isE^", "Pound", "might", "blA^ck", "PR^ivY^"]);
 
 =back
 
 This is a wrapper to L<wordlist> designed to be a convenient helper to solve
 Wordle puzzle. By default it greps from the C<EN::Wordle> wordlist. It accepts
-5-character pattern like this:
+a series of guesses in a format like the following:
 
- wt_S_
+ A^R^isE^
+ Pound
+ might
+ blA^ck
+ PR^ivY^
 
-where lowercase means letter in the incorrect position, uppercase means letter
-in the correct position, and underscore means unguessed. It will convert this
-pattern to regex and the C<--chars-unordered> option and pass it to C<wordlist>.
+where lowercase means wrong guess, uppercase means correct letter and position,
+while (uppercase) letter followed by a caret (C<^>) means the letter exists in
+another position. It will convert these guesses to regex patterns and the
+C<--chars-unordered> option and pass it to C<wordlist>.
 
 This function is not exported.
 
